/****
 *
 * Ttulo: MDC
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 22/04/2012
 * ltima modificao: 22/04/2012
 *
 * Descrio: Calcula o MDC de dois nmeros inteiros positivos
 *
 * Entrada: Valores inteiros positivos ou zero.
 *          s/n ou S/N para continuar ou encerrar o programa
 *
 * Sada: v. exemplo de execuo
 *
 ****/

#include <stdio.h>   /* Entrada e sada */
#include "leitura.h" /* LeituraFacil    */

/****
 *
 * LeOpcaoSimNao(): L uma opo do tipo sim/no
 *
 * Parmetros: Nenhum
 *
 * Retorno: 1, se a opo for 's' ou 'S'
 *          0, se a opo for 'n' ou 'N'
 *
 ****/
int LeOpcaoSimNao(void)
{
   int op;

   op = LeOpcao("sSnN");

   if (op == 's' || op == 'S') {
      return 1;
   }

   return 0;
}

/****
 *
 * LeNatural(): L um nmero natural (i.e., um nmero
 *              inteiro no negativo)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNatural(void)
{
   int numNatural; /* O nmero a ser lido */

   numNatural = LeInteiro(); /* Primeira tentativa */

   while(numNatural < 0) {
      printf( "\a\n\t>>> Valor invalido. "
              "Tente novamente <<<\n\t> " );
      numNatural = LeInteiro(); /* Faz nova tentativa */
   }

   return numNatural;
}

/****
 *
 * MDC(): Calcula o MDC de dois nmeros inteiros positivos
 *
 * Parmetros:
 *      x, y (entrada): nmeros cujo MDC ser calculado
 *
 * Retorno: O referido MDC, se x e y forem positivos.
 *          Zero, em caso contrrio.
 *
 ****/
int MDC(int x, int  y)
{
   int divisor;

      /* MDC  definido apenas para */
      /* nmeros inteiros positivos */
   if ( (x <= 0) || (y <= 0) )
      return 0; /* 0 no  um MDC vlido. Este valor */
                /* indica uma condio de erro.      */

   /*                                                   */
   /* O MDC de dois nmeros  no mximo igual ao menor  */
   /* dos dois nmeros. Portanto, faz-se o valor        */
   /* inicial do divisor o menor deles. Ento, enquanto */
   /* 'divisor' no divide ambos, reduz-se seu valor de */
   /* uma unidade. Se os nmeros forem primos entre si, */
   /* 'divisor' atingir o  valor 1, que  divisor de   */
   /* qualquer nmero inteiro positivo.                 */
   /*                                                   */

      /* Atribui a 'divisor' o menor dos parmetros */
   divisor = (y < x) ? y : x;

      /* Enquanto 'divisor' no assumir um valor que divida */
      /* tanto x quanto y, continua decrementando seu valor */
   while ( (x%divisor) || (y%divisor) ) {
      divisor--;
   }

   return divisor;
}

/****
 *
 * main(): L continuamente dois valores inteiros no negativos,
 *         calcula o MDC deles e apresenta o resultado
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int a, b; /* Dois valores que tero o MDC calculado */

      /* Apresenta o programa */
   printf("\n\t>>> Este programa calcula o MDC de"
          "\n\t>>> dois numeros inteiros positivos.\n");

      /* O lao encerra quando o usurio escolhe 'n' ou 'N' */
   do {
         /* L dois valores no negativos */
      printf("\n>>> Digite um numero inteiro que "
             "nao seja negativo:\n\t> ");
      a = LeNatural();

      printf("\n>>> Digite outro numero inteiro que "
             "nao seja negativo:\n\t> ");
      b = LeNatural();

         /* Se um dos dois valores for 0, informa   */
         /* que no h MDC. Caso contrrio, calcula */
         /* o MDC e o apresenta na tela.            */
      if (!a || !b) {
         printf("\n\t>>> Nao existe MDC de: %d e %d", a, b);
      } else {
         printf("\n\t>>> O MDC e': %d\n", MDC(a, b));
      }

         /* Verifica se o usurio deseja continuar */
      printf("\n\t>>> Deseja continuar (s/n)? ");
   } while (LeOpcaoSimNao());

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa calcula o MDC de
        >>> dois numeros inteiros positivos.

>>> Digite um numero inteiro que nao seja negativo:
        > 7

>>> Digite outro numero inteiro que nao seja negativo:
        > 28

        >>> O MDC e': 7

        >>> Deseja continuar (s/n)? s

>>> Digite um numero inteiro que nao seja negativo:
        > 4

>>> Digite outro numero inteiro que nao seja negativo:
        > 16

        >>> O MDC e': 4

        >>> Deseja continuar (s/n)? n

        >>> Obrigado por usar este programa.

***********************************************************/
