/****
 *
 * Ttulo: Calculando Raiz Quadrada por meio de
 *         Busca Binria
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Calcula a raz quadrada de um nmero
 *            real no negativo usando busca binria
 *
 * Entrada: Um nmero real
 *
 * Sada: A raiz quadrada do nmero
 *
 ****/

#include <stdio.h>   /* printf() */
#include <math.h>    /* sqrt()   */
#include "leitura.h" /* LeReal() */

   /* Tolerncia de erro admitida no clculo da raiz */
#define TOLERANCIA 1.0E-14

/****
 *
 * RaizQuadrada(): Calcula a raiz quadrada de um nmero
 *                 real no negativo usando busca binria
 *
 * Parmetros:
 *      x (entrada): nmero cuja raiz ser calculada
 *      inferior (entrada): limite inferior do intervalo
 *      superior (entrada): limite superior do intervalo
 *
 * Retorno: A raiz quadrada positiva do primeiro parmetro,
 *          se inferior <= superior e o nmero no for
 *          negativo. Caso contrrio, um valor negativo
 *          que indica ocorrncia de erro.
 *
 ****/
double RaizQuadrada(double x, double inferior, double superior)
{
   double meio; /* Valor mdio do intervalo */

      /* Se o limite inferior  maior do que o limite */
      /* superior ou o valor que se deseja calcular a */
      /* raiz  negativo, esta funo foi chamada     */
      /* incorretamente. Este  o significado do      */
      /* valor retornado neste caso.                  */
   if (inferior > superior || x < 0) {
      return -1.0;
   }

      /* O lao encerra quando a largura do intervalo */
      /* tornar-se menor do que a tolerncia de erro  */
   while (1) {
         /* Calcula o meio do intervalo */
      meio =(inferior + superior)/2.0;

         /* Verifica se a raiz quadrada foi encontrada  */
         /* de acordo com a tolerncia de erro admitida */
      if (superior - inferior <= TOLERANCIA) {
         break; /* A diferena entre 'superior' e 'inferior'  */
                /* to pequena que se pode considerar o meio   */
                /* deste intervalo como sendo a raiz procurada */
      }

      /********************************/
      /* >>> Reduo do intervalo <<< */
      /********************************/

         /* Se o quadrado do meio for maior do que    */
         /* o nmero, a raiz est na metade inferior  */
         /* do intervalo. Caso contrrio, a raiz est */
         /* na metade superior do intervalo.          */
      if (meio*meio > x) {
         superior = meio; /* A raiz est na metade inferior */
      } else {
         inferior = meio; /* A raiz est na metade superior */
      }
   }

      /* A varivel 'meio' armazena a raiz procurada, */
      /* de acordo com a tolerncia admitida          */
   return meio;
}

/****
 *
 * main(): Testa RaizQuadrada() comparando o resultado apresentado
 *         por esta funo com aquele apresentado por sqrt()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   double num;

      /* Apresenta o programa */
   printf("\n\t>>> Este programa calcula a raiz quadrada de um"
          "\n\t>>> numero nao negativo usando busca binaria.\n");

      /* L o nmero cuja raiz quadrada ser calculada */
   printf("\n\t>>> Digite um numero nao negativo> ");
   num = LeReal();

      /* Se a raiz quadrada existir, calcula-a e exibe-a */
   if(num < 0) { /* Usurio desobediente */
      printf("\n\t>>> Nao ha raiz quadrada de numero negativo\n");
   } else { /* Usurio bacana */
      printf( "\n\t>>> Raiz quadrada aproximada de %3.2f: ", num );

         /* Determina o intervalo no qual se encontra a  */
         /* raiz quadrada, chama a funo RaizQuadrada() */
         /* de acordo com o intervalo e exibe resultado  */
      if(num < 1) { /* A raiz est entre 'num' e 1.0 */
         printf( "%3.2f\n", RaizQuadrada(num, num, 1.0) );
      } else { /* A raiz est entre 1.0 e 'num' */

         printf( "%3.2f\n", RaizQuadrada(num, 1.0, num) );
      }

         /* Apresenta o resultado calculado usando sqrt()*/
      printf( "\t>>> Raiz quadrada de %3.2f "
              "usando sqrt(): %3.2f\n", num, sqrt(num) );
   }

   return 0;
}

/******************* Exemplo de Execuo *******************

       >>> Este programa calcula a raiz quadrada de um
       >>> numero nao negativo usando busca binaria.

        >>> Digite um numero nao negativo> 2

        >>> Raiz quadrada aproximada de 2.00: 1.41
        >>> Raiz quadrada de 2.00 usando sqrt(): 1.41

***********************************************************/
