/****
 *
 * Ttulo: Nmeros Primos II
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 02/06/2012
 * ltima modificao: 02/06/2012
 *
 * Descrio: Determina se nmeros inteiros positivos
 *            introduzidos via teclado so primo
 *
 * Entrada: Valores inteiros positivos ou zero
 *          para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

#include <stdio.h>   /* printf()    */
#include <math.h>    /* sqrt()      */
#include "leitura.h" /* LeInteiro() */

/****
 *
 * LeNatural(): L um nmero natural (i.e., um nmero
 *              inteiro no negativo)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNatural(void)
{
   int numNatural; /* O nmero a ser lido */

inicio: /* Volta para c se o valor for invlido */
   numNatural = LeInteiro();

   while(numNatural < 0) {
      printf( "\a\n\t>>> Valor invalido. "
              "Tente novamente <<<\n\t> " );
      goto inicio; /* No faz mal a ningum */
   }

   return numNatural;
}

/****
 *
 * EhPrimo2(): Verifica se um nmero inteiro
 *             maior do que um  primo ou no
 *
 * Parmetros:
 *      n (entrada): o nmero que ser testado
 *
 * Retorno: 1, se o nmero for primo
 *          0, se o nmero no for primo
 *         -1, se for indefinido (i.e., se n <= 1)
 *
 ****/
int EhPrimo2(int n)
{
   int i, raiz;

      /* O conceito de nmero primo no  definido */
      /* para nmeros inteiros menores do que dois */
   if (n <= 1) {
      return -1; /* Indefinido */
   }

   /*                                                */
   /* No mximo, um nmero  divisvel por sua raiz  */
   /* quadrada. Portanto, se no for encontrado um   */
   /* divisor para o parmetro no mximo igual a sua */
   /* raiz, a busca por um divisor  encerrada. Para */
   /* evitar que essa raiz seja recalculada a cada   */
   /* passagem no lao abaixo, armazena-se esse      */
   /* valor numa varivel local.                     */
   /*                                                */

      /* Calcula a raiz quadrada do nmero a ser testado */
   raiz = sqrt(n);

      /* Verifica se o nmero tem algum divisor. No  */
      /* mximo, um nmero  divisvel por sua raiz. */
   for (i = 2; i <= raiz; ++i) {
      if (!(n%i)) {
         return 0; /* Encontrado um divisor */
      }
   }

      /* No foi encontrado nenhum divisor para */
      /* o nmero dado. Portanto, ele  primo.  */
   return 1;
}

/****
 *
 * main(): Determina se nmeros inteiros positivos
 *         introduzidos via teclado so primo
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int num;

      /* Apresenta o programa e explica seu funcionamento */
   printf( "\n\t>>> Este programa verifica se numeros inteiros"
           "\n\t>>> nao-negativos sao primos ou nao. "
           "\n\t>>> Para encerra-lo, digite zero ou um.\n\n" );

      /* O lao principal do programa encerra */
      /* quando o usurio introduz 0 ou 1     */
   while (1) {
      printf("\n\t>>> Digite um numero inteiro que "
             "nao seja negativo:\n\t> ");
      num = LeNatural(); /* L o nmero */

      if (num <= 1) { /* Encerra o lao */
         break;
      }

         /* Verifica se o nmero  primo ou no */
         /* e apresenta a mensagem apropriada   */
      printf( "\n\t>>> %d %s e' primo\n", num,
              EhPrimo2(num) ? "" : "nao" );
   }

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa verifica se numeros
        >>> inteiros nao-negativos sao primos
        >>> ou nao. Para encerra-lo, digite
        >>> zero ou um.


        >>> Digite um numero inteiro que nao seja negativo:
        > -1

        >>> Valor invalido. Tente novamente <<<
        > 56

        >>> 56 nao e' primo

        >>> Digite um numero inteiro que nao seja negativo:
        > 991

        >>> 991  e' primo

        >>> Digite um numero inteiro que nao seja negativo:
        > 0

        >>> Obrigado por usar este programa.

***********************************************************/
