/****
 *
 * Ttulo: Leitura de Datas com Validao I
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 22/06/2012
 * ltima modificao: 22/06/2012
 *
 * Descrio: L uma data e verifica sua validade
 *
 * Entrada: Valores inteiros introduzidos via teclado
 *          representando dia, mes e ano
 *
 * Sada: Uma mensagem indicando se a data  vlida ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* printf()    */
#include "leitura.h" /* LeInteiro() */

/***************** Constantes Simblicas ******************/

#define PRIMEIRO_ANO_BISSEXTO 1752

/************************* Aluses ************************/

extern int LeNatural(void);
extern int EhAnoBissexto(int ano);
extern int EhDataValida(int dia, int mes, int ano);

/***************** Definies de Funes ******************/

/****
 *
 * LeNatural(): L um nmero natural (i.e., um nmero
 *              inteiro no negativo)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNatural(void)
{
   int numNatural; /* O nmero a ser lido */

inicio: /* Volta para c se o valor for invlido */
   numNatural = LeInteiro();

   while(numNatural < 0) {
      printf( "\a\n\t>>> Valor invalido. "
              "Tente novamente <<<\n\t> " );
      goto inicio; /* No faz mal a ningum */
   }

   return numNatural;
}

/****
 *
 * EhAnoBissexto(): Verifica se um ano  bissexto
 *
 * Parmetros: ano (entrada): o ano que ser testado
 *
 * Retorno: 1, se o ano for bissexto
 *          0, se o ano no for bissexto
 *
 * Observao: Um ano  bissexto quando:
 *             * Ele  mltiplo de 400 ou
 *             * Ele  mltiplo de 4, mas no  mltiplo de 100
 *
 ****/
int EhAnoBissexto(int ano)
{
      /* Se o ano for anterior ao primeiro ano considerado */
      /* bissexto, ele no pode ser assim considerado      */
   if (ano < PRIMEIRO_ANO_BISSEXTO) {
      return 0;
   }

   return !(ano%400) || (!(ano%4) && ano%100);
}

/****
 *
 * EhDataValida(): verifica se uma data  vlida
 *
 * Parmetros: dia (entrada) - o dia
 *             mes (entrada) - o ms
 *             ano (entrada) - o ano
 *
 * Retorno: 1, se a data for vlida; 0, em caso contrrio
 *
 ****/
int EhDataValida(int dia, int mes, int ano)
{
      /* No existe dia menor do que ou igual */
      /* a zero, nem dia maior do que 31      */
   if ( dia <= 0 || dia > 31) {
      return 0; /* Dia invlido */
   }

      /* Verifica se o dia  vlido, o que depende do ms */
   switch(mes) {
      case 1: /* Estes meses sempre tm 31 dias */
      case 3:
      case 5:
      case 7:
      case 8:
      case 10:
      case 12:
         return 1;
      case 4: /* Estes meses sempre tm 30 dias */
      case 6:
      case 9:
      case 11:
            /* Se o ms tiver mais de 30 */
            /* dias, ele  invlido      */
         if (dia > 30) {
            return 0;
         } else {
            return 1;
         }
      case 2: /* Este ms pode ter 28 ou 29 dias */
            /* Se o ms tiver mais de 29 */
            /* dias, ele  invlido      */
         if (dia > 29) {
            return 0;
         } else if (dia < 29) {
            return 1; /* Ms tem 28 dias */
         } else if (EhAnoBissexto(ano)) {
            return 1; /* Ms tem 29 dias e ano  bissexto */
         } else {
            return 0;
         }
      default: /* A execuo no deve chegar at aqui */
         printf("\nOcorreu um erro sobrenatural");
   }

      /* A execuo tambm no se deve chegar at aqui */
   printf("\nOcorreu outro erro sobrenatural");

   return 0; /* Esta instruo nunca deve ser executada */
}

/****
 *
 * main(): L uma data e verifica sua validade
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int d, m, a; /* Dia, ms e ano, respectivamente */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa verifica a validade de"
           "\n\t>>> uma data introduzida pelo usuario.\n" );

   printf("\n\t>>> Digite o dia:\n\t> ");
   d = LeNatural();

   printf("\n\t>>> Digite o mes:\n\t> ");
   m = LeNatural();

   printf("\n\t>>> Digite o ano:\n\t> ");
   a = LeNatural();

      /* Apresenta o resultado da validao de acordo com */
      /* o valor retornado pela funo EhDataValida()     */
   printf( "\n\t>>> A data %.2d/%.2d/%.2d %se\' valida\n",
           d, m, a, EhDataValida(d, m, a) ? "" : "NAO " );

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa verifica a validade de
        >>> uma data introduzida pelo usuario.

        >>> Digite o dia:
        > 29

        >>> Digite o mes:
        > 2

        >>> Digite o ano:
        > 2000

        >>> A data 29/02/2000 e' valida

**********************************************************/
