/****
 *
 * Ttulo: Conjectura de Collatz II
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/03/2013
 * ltima modificao: 16/03/2013
 *
 * Descrio: Encontra o nmero entre 1 e MAX que
 *            produz a maior sequncia de Collatz
 *
 * Entrada: No h
 *
 * Sada: Uma mensagem informando qual  o nmero que produz
 *        a maior sequncia de Collatz
 *
 * Definio: Seja n um inteiro positivo. Construa uma
 *            sequncia tendo n como primeiro termo e tal
 *            que cada termo seguinte  obtido dividindo-se
 *            o termo corrente por 2, se ele for par, ou
 *            multiplicando-se por 3 e somando-se 1, se ele
 *            for mpar. A Conjectura de Collatz prope que
 *            eventualmente um termo da sequncia assume o
 *            valor 1. Esta proposio nunca foi provada
 *            como verdadeira ou falsa (por isso 
 *            considerada uma conjectura).
 *
 ****/

#include <stdio.h> /* printf() */

   /* Maior valor que pode ser usado como primeiro */
   /* termo de uma sequncia de Collatz            */
#define MAX 100000

/***************** Definies de Funes ******************/

/****
 *
 * Collatz2(): Determina o nmero de termos da sequncia de
 *             Collatz que comea com um dado inteiro positivo
 *
 * Parmetros: n (entrada) - termo inicial da sequncia
 *
 * Retorno: O nmero de termos da sequncia
 *
 ****/
int Collatz2(int n)
{
   int cont = 1; /* Conta o nmero de termos da sequncia e */
                 /*  iniciado com 1 porque a sequncia tem */
                 /* pelo menos um termo, que  o parmetro  */

      /* O primeiro termo deve ser positivo */
   if (n <= 0) {
      return 0; /* No existe sequncia */
   }

      /* Determina os demais termos. O lao encerra */
      /* quando  encontrado um termo igual a 1.    */
   while (1) {
         /* Gera o prximo termo */
      if (!(n%2)) { /* O termo corrente  par */
         n = n/2; /* Calcula o prximo termo */
      } else { /* O termo corrente  mpar */
         n = 3*n + 1; /* Calcula o prximo termo */
      }

      ++cont; /* Mais um termo foi gerado */

         /* Se o novo termo for igual a 1,  */
         /* chegou-se ao final da sequncia */
      if (n == 1) {
         break; /* Encerra o lao */
      }
   }

   return cont;
}

/****
 *
 * main(): Encontra o nmero entre 1 e MAX que produz a maior
 *         sequncia de Collatz
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int nMax, /* Nmero que gera a maior sequncia */
       maxTermos, /* Nmero de termos da maior sequncia */
       nTermos, /* Nmero de termos de cada sequncia */
       i;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa encontra o numero entre"
           "\n\t>>> 1 e %d que produz a maior sequencia"
           "\n\t>>> de Collatz.\n", MAX );

      /* Inicia as variveis 'nMax' e 'maxTermos' com 1 */
   nMax = 1;
   maxTermos = 1; /* A sequncia que comea   */
                  /* com 1 tem apenas 1 termo */

      /* Determina os nmeros de termos das sequncias  */
      /* comeando com 2 at MAX e verifica qual desses */
      /* valores produz a maior sequncia               */
   for (i = 2; i <= MAX; ++i) {
         /* Determina o nmero de termos */
         /* da sequncia corrente        */
      nTermos = Collatz2(i);

         /* Verifica se o nmero de termos da sequncia */
         /* corrente  maior do que o maior valor atual */
      if (nTermos > maxTermos) {
            /* A sequncia corrente possui */
            /* o maior nmero de termos    */
         maxTermos = nTermos;

         nMax = i; /* i produz a maior sequncia at aqui */
      }
   }

      /* Apresenta o nmero que produz a maior sequncia */
   printf( "\n\t>>> O numero entre 1 e %d que produz a"
           "\n\t>>> maior sequencia de Collatz e' %d.\n",
           MAX, nMax );

      /* Apresenta o nmero de termos da maior sequncia */
   printf( "\n\t>>> Essa sequencia possui %d termos.\n",
           maxTermos );

   return 0;
}

/****************** Resultado do Programa *****************

        >>> Este programa encontra o numero entre
        >>> 1 e 100000 que produz a maior sequencia
        >>> de Collatz.

        >>> O numero entre 1 e 100000 que produz a
        >>> maior sequencia de Collatz e' 77031.

        >>> Essa sequencia possui 351 termos.

**********************************************************/
