/****
 *
 * Ttulo: Qual  o Troco?
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 28/03/2013
 * ltima modificao: 02/04/2013
 *
 * Descrio: Apresenta na tela os nmeros de cdulas e
 *            moedas que devem ser entregues a um cliente
 *            como troco de uma compra
 *
 * Entrada: Valores de compras e valores pagos pelos clientes
 *
 * Sada: Os nmeros de cdulas e moedas que devem ser cedidas ao
 *        cliente como troco aps cada compra e, ao final do
 *        programa, o nmero total de cdulas e moedas utilizadas
 *
 * Observao: Por simplicidade, R$1  considerado como cdula,
 *             e no como moeda
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada */
#include <math.h>    /* Funo fabs()   */
#include "leitura.h" /* LeituraFacil    */

/***************** Constantes Simblicas ******************/

#define MAIOR_TROCO 99.99  /* Maior troco recomendado */
                           /* por este programa       */
#define N_CEDULAS  6      /* Nmero de tipos de cdulas */
#define N_MOEDAS   4      /* Nmero de tipos de moedas  */
   /* Preciso utilizada na comparao de nmeros     */
   /* reais. Esse valor tambm  adicionado a valores */
   /* reais para evitar erros de truncamento na       */
   /* converso desses valores para inteiros.         */
#define DELTA 1.0E-14

/***************** Definies de Funes ******************/

/****
 *
 * ComparaDoubles(): Compara dois valores do tipo double
 *
 * Parmetros:
 *      d1 (entrada): primeiro valor
 *      d2 (entrada): segundo valor
 *
 * Retorno: 0, se os nmeros forem considerados iguais
 *          -1, se d1 for considerado menor do que d2
 *          1, se d1 for considerado maior do que d2
 *
 ****/
int ComparaDoubles(double d1, double d2)
{
   /****************************************************/
   /*                                                  */
   /* Verifica se o valor absoluto da diferena entre  */
   /* os nmeros  menor do que ou igual  preciso    */
   /* determinada pela constante DELTA. Se for o caso, */
   /* eles so considerados iguais. Caso contrrio,    */
   /* verifica qual deles  menor ou maior.            */
   /*                                                  */
   /****************************************************/

   if (fabs(d1 - d2) <= DELTA) {
      return 0; /* d1 e d2 so considerados iguais */
   } else if (d1 < d2) {
      return -1; /* d1  menor do que d2 */
   } else {
      return 1; /* d1  maior do que d2 */
   }
}

/****
 *
 * main(): Apresenta na tela os nmeros de cdulas e moedas que
 *         devem ser cedidas a clientes como trocos de compras
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   double valorCompra, /* Valor da compra */
          pago, /* Valor pago pelo cliente */
          troco, /* Valor do troco */
          trocoCorrigido; /* Valor do troco corrigido */
                   /* para evitar erro de truncamento */
             /* Array que contm as cdulas disponveis */
   int    cedulas[N_CEDULAS] = {50, 20, 10, 5, 2, 1},
            /* Array que contm as moedas disponveis */
          moedas[N_MOEDAS] = {50, 25, 10, 5},
            /* Total de cdulas de cada tipo que forem    */
            /* entregues durante uma execuo do programa */
          totalCedulas[N_CEDULAS] = {0},
            /* Total de moedas de cada tipo que forem     */
            /* entregues durante uma execuo do programa */
          totalMoedas[N_MOEDAS] = {0},
          parcial, /* Nmero parcial de cdulas/moedas */
                   /* a serem entregues ao cliente     */
          pInteira,     /* Parte inteira do troco     */
          pFracionaria, /* Parte fracionria do troco */
          i;

      /**************************************************/
      /* Os arrays cedulas[] e moedas[] foram ordenados */
      /* em ordem decrescente para que o troco reunisse */
      /* o menor nmero possvel de cdulas e moedas    */
      /**************************************************/

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa apresenta numeros de cedulas"
           "\n\t>>> ou moedas que devem ser entregues a clientes"
           "\n\t>>> como trocos de compras. Digite 0 como valor"
           "\n\t>>> da compra para encerrar o programa." );

      /* O lao encerra quando o usurio introduzir 0 */
      /* como valor de uma compra                     */
   while (1) {
         /* L o valor da compra */
      printf("\n\n\t>>> Valor da compra (R$)? ");
      valorCompra = LeReal();

         /* Se o usurio digitou 0, encerra o lao */
      if (!ComparaDoubles(valorCompra, 0.0)) {
         break;
      }

         /* L o valor pago pelo cliente */
      printf("\t>>> Valor pago (R$)? ");
      pago = LeReal();

         /* Calcula o valor do troco */
      troco = pago - valorCompra;

         /* Verifica se h troco a ser entregue */
      if (ComparaDoubles(troco, 0.0) <= 0) {
         printf("\a\n\t>>> Nao ha' troco!\n");
      } else if (ComparaDoubles(troco, MAIOR_TROCO) > 0) {
            /* O valor pago excede limite mximo de troco */
         printf("\n\t>>> O valor pago deveria ser menor\n");
      } else { /* H troco a ser entregue */

         /*                                              */
         /* Agora, deve-se calcular as partes inteira e  */
         /* fracionria do troco. Para evitar que uma    */
         /* varivel inteira receba um valor incorreto   */
         /* por causa de truncamento de um valor real,   */
         /* deve-se arredondar o valor real.Para obter o */
         /* efeito desejado sem introduzir um novo erro, */
         /* acrescenta-se um pequeno valor (DELTA) ao    */
         /* valor real antes que ele seja convertido.    */
         /*                                              */

         trocoCorrigido = troco + DELTA;

            /* Calcula as partes inteira */
            /* e fracionria do troco    */
         pInteira = trocoCorrigido;
         pFracionaria = (trocoCorrigido - pInteira)*100;

            /* Apresenta o valor do troco */
         printf("\n\t>>> Troco: R$%0.2f <<<\n", troco);

            /* Determina e apresenta na tela o nmero  */
            /* de cdulas a serem entregues ao cliente */
         for (i = 0; i < N_CEDULAS; ++i) {
               /* Calcula o nmero de cdulas com o */
               /* valor determinado por cedulas[i]  */
            parcial = pInteira/cedulas[i];

               /* Se houver cdulas com o valor dado por  */
               /* cedulas[i], apresenta-as na tela e reduz*/
               /* a parte inteira do valor j fornecido   */
            if (parcial) {
               printf( "\n\t>>> %d cedula%s de R$%d",
                       parcial, parcial > 1 ? "s" : "",
                       cedulas[i] );

                  /* Atualiza o nmero total de cdulas */
                  /* entregues do tipo armazenado no    */
                  /* ndice i do array cedulas[]        */
               totalCedulas[i] =  totalCedulas[i] + parcial;

                  /* A parte inteira do troco precisa  */
                  /* ser reduzida do valor apresentado */
               pInteira = pInteira - parcial*cedulas[i];
            }
         }

            /* Determina e apresenta na tela o nmero */
            /* de moedas a serem entregues ao cliente */
         for (i = 0; i < N_MOEDAS; ++i) {
            parcial = pFracionaria/moedas[i];

               /* Se houver moedas com o valor dado por     */
               /* moedas[i], apresenta-as na tela e reduz a */
               /* parte fracionria do valor j fornecido   */
            if (parcial) {
               printf("\n\t>>> %d moeda%s de %d centavos",
                      parcial, parcial > 1 ? "s" : "", moedas[i]);

                  /* Atualiza o nmero total de moedas */
                  /* entregues do tipo armazenado no   */
                  /* ndice i do array moedas[]        */
               totalMoedas[i] =  totalMoedas[i] + parcial;

                  /* A parte fracionria do troco precisa */
                  /* ser reduzida do valor apresentado    */
               pFracionaria = pFracionaria - parcial*moedas[i];
            }
         }

            /* Se sobraram alguns centavos cujos valores  */
            /* so menores do que o menor valor de moeda  */
            /* e o cliente exigir, sugere que o atendente */
            /* entregue mais uma moeda com o menor valor  */
         if (pFracionaria) {
            printf("\n\t>>> Se o cliente exigir, de-lhe"
                    "\n\t    mais uma moeda de %d centavos",
                    moedas[--i]);
         }
      }
   }

         /*                                          */
         /* Apresenta um resumo das cdulas e moedas */
         /* entregues durante a execuo do programa */
         /*                                          */

   printf( "\n>>> Total de cedulas entregues <<<\n" );

   for (i = 0; i < N_CEDULAS; ++i) {
         /* Se o nmero total de cdulas do tipo armazenado */
         /* no ndice i do array cedulas[] for diferente de */
         /* 0, apresenta esse total                         */
      if (totalCedulas[i]) {
         printf( "\n\t>>> %d cedula%s de R$%d", totalCedulas[i],
                 totalCedulas[i] > 1 ? "s" : "", cedulas[i] );
      }
   }

   printf( "\n\n>>> Total de moedas entregues <<<\n" );

   for (i = 0; i < N_MOEDAS; ++i) {
         /* Se o nmero total de moedas do tipo armazenado */
         /* no ndice i do array moedas[] for diferente de */
         /* 0, apresenta esse total                        */
      if (totalMoedas[i]) {
         printf( "\n\t>>> %d moeda%s de R$%d", totalMoedas[i],
                 totalMoedas[i] > 1 ? "s" : "", moedas[i] );
      }
   }

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa apresenta numeros de cedulas
        >>> ou moedas que devem ser entregues a clientes
        >>> como trocos de compras. Digite 0 como valor
        >>> da compra para encerrar o programa.

        >>> Valor da compra (R$)? 9.90
        >>> Valor pago (R$)? 10

        >>> Troco: R$0.10 <<<

        >>> 1 moeda de 10 centavos

        >>> Valor da compra (R$)? 25.6
        >>> Valor pago (R$)? 30

        >>> Troco: R$4.40 <<<

        >>> 2 cedulas de R$2
        >>> 1 moeda de 25 centavos
        >>> 1 moeda de 10 centavos
        >>> 1 moeda de 5 centavos

        >>> Valor da compra (R$)? 0

>>> Total de cedulas entregues <<<

        >>> 2 cedulas de R$2

>>> Total de moedas entregues <<<

        >>> 1 moeda de R$25
        >>> 2 moedas de R$10
        >>> 1 moeda de R$5

**********************************************************/
