/****
 *
 * Ttulo: Inserindo um String em Outro
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Insere strings em outros strings nas posies indicadas
 *
 * Entrada: Strings e nmeros inteiros que indicam posies de
 *          insero
 *
 * Sada: Os strings com caracteres inseridos e eventuais mensagens
 *        informando sobre entradas de dados incorretas
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* printf()                 */
#include <string.h>  /* strlen() e strcat()      */
#include "leitura.h" /* LeString() e LeInteiro() */

/***************** Constantes Simblicas ******************/

   /* Tamanhos dos arrays que armazenaro os strings */
#define TAM_ARRAY 30

/***************** Definies de Funes ******************/

/****
 *
 * InsereString(): Insere um string em outro
 *
 * Parmetros:
 *     str1 (entrada/sada) - string que receber a insero
 *     str2 (entrada) - string a ser inserido
 *     local (entrada) - ndice do elemento do string str1
 *                       no qual comea a insero
 *
 * Retorno: Endereo do string resultante
 *
 * Observao: Se o local especificado para insero for maior do
 *             que o nmero de caracteres do string de origem,
 *             esta funo funciona como strcat()
 *
 ****/
char *InsereString(char *str1, const char *str2, int local)
{
   int   tamanho1, /* Tamanho do string que recebe caracteres */
         tamanho2, /* Nmero de caracteres que sero doados */
         i;

      /* Calcula o tamanho do string no */
      /* qual ser feita a insero     */
   tamanho1 = strlen(str1);

      /* Um string s  indexado de 0 a strlen(s). Portanto, se */
      /* o local da insero for maior do que ou igual ao seu  */
      /* tamanho deixa-se strcat() completar o servio.        */
   if (local >= tamanho1) {
      return strcat(str1, str2);
   }

      /* Calcula o nmero de caracteres que sero inseridos */
   tamanho2 = strlen(str2);

      /* Desloca para adiante cada caractere do string de      */
      /* destino um nmero de posies igual ao nmero de      */
      /* caracteres que sero inseridos,  a partir do local de */
      /* insero. Os ltimos caracteres so deslocado antes.  */
   for (i = tamanho1 + 1; i >= local; --i) {
      str1[i + tamanho2] = str1[i];
   }

      /* O espao para insero est aberto. Resta     */
      /* copiar os caracteres que devem ser inseridos. */
   for (i = 0; i < tamanho2; ++i) {
      str1[i + local] = str2[i];
   }

   return str1;
}

/****
 *
 * main(): Insere strings em outros strings nas posies indicadas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char str1[TAM_ARRAY], str2[TAM_ARRAY];
   int  posicao;

      /* Apresenta o programa */
   printf("\n\t>>> Este programa insere caracteres numa"
          "\n\t>>> cadeia de caracteres numa dada posicao."
          "\n\t>>> Digite [ENTER] como primeira cadeia de"
          "\n\t>>> caracteres para encerrar o programa.\n");

      /* O lao encerra quando for lido apenas */
      /* [ENTER] como primeiro string          */
   while (1) {
         /* L o primeiro string */
      printf("\n\t>>> Digite uma cadeia de caracteres "
             "\n\t    (max = %d digitos): ", TAM_ARRAY - 1);
      (void) LeString(str1, TAM_ARRAY);

         /* Se o string for vazio encerra o lao */
      if (!*str1) { /* Usurio digitou apenas [ENTER] */
         break;
      }

         /* L o string a ser inserido */
      printf("\n\t>>> Digite os caracteres a ser inseridos "
             "\n\t    (max = %d digitos): ", TAM_ARRAY - 1);
      (void) LeString(str2, TAM_ARRAY);

         /* O lao encerra quando o usurio   */
         /* digitar um valor inteiro positivo */
      while (1) {
         printf( "\n\t>>> Digite a posicao de insercao: " );
         posicao = LeInteiro();

            /* Usurio comum no indexa strings a partir de   */
            /* zero, como faz a funo InsereString(). Assim, */
            /*  necessrio decrementar a posio introduzida */
         --posicao;

         if (posicao > 0) {
            break;
         }

         printf("\a\n\t>> O valor deve ser positivo <<\n");
      }

         /* Insere os caracteres de 'str2' em */
         /* 'str1' na posio especificada    */
      InsereString(str1, str2, posicao);

         /* Apresenta o resultado da operao */
      printf( "\n\t>>> Resultado da insercao: \"%s\"\n",
              str1 );
   }

   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa insere caracteres numa
        >>> cadeia de caracteres numa dada posicao.
        >>> Digite [ENTER] como primeira cadeia de
        >>> caracteres para encerrar o programa.

        >>> Digite uma cadeia de caracteres
            (max = 29 digitos): bocha

        >>> Digite os caracteres a ser inseridos
            (max = 29 digitos): la

        >>> Digite a posicao de insercao: 3

        >>> Resultado da insercao: "bolacha"

        >>> Digite uma cadeia de caracteres
            (max = 29 digitos): bolacha

        >>> Digite os caracteres a ser inseridos
            (max = 29 digitos): da

        >>> Digite a posicao de insercao: 10

        >>> Resultado da insercao: "bolachada"

        >>> Digite uma cadeia de caracteres
            (max = 29 digitos): [ENTER]

       >>> Obrigado por usar este programa.

**********************************************************/
