/****
 *
 * Ttulo: Arte Moderna na Tela (do Computador)
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 26/06/2012
 * ltima modificao: 26/06/2012
 *
 * Descrio: Desenha aleatoriamente na tela caracteres
 *            que representam cores
 *
 * Entrada: Nada
 *
 * Sada: Um desenho com configurao aleatria
 *
 ****/

/************************ Includes ************************/

#include <stdlib.h> /* srand() e rand() */
#include <stdio.h>  /* putchar()        */
#include <time.h>   /* time()           */

/****************** Constantes Simblicas *****************/

#define HORIZONTAL 18 /* Tamanho horizontal do quadro */
#define VERTICAL   12 /* Tamanho vertical do quadro   */

/******************* Definies de Tipos ******************/

   /* Tipo de enumerao que define */
   /* as cores que sero usadas     */
typedef enum { BRANCO = ' ', AZUL = '+',
               VERDE = '.', VERMELHO = '\'' } tPaleta;

/****************** Definies de Funes *****************/

/****
 *
 * EscolheCor(): Escolhe a cor da prxima pincelada
 *
 * Parmetros: Nenhum
 *
 * Retorno: A primeira cor sorteada por rand() que coincide
 *          com uma das constantes do tipo tPaleta
 *
 ****/
tPaleta EscolheCor(void)
{
   tPaleta cor;

      /* Chama a funo rand() repetidamente at que o */
      /* valor retornado por esta funo coincida com  */
      /* um dos valores da enumerao do tipo tPaleta  */
   do {
      cor = rand();
   } while ( cor != VERMELHO && cor != AZUL &&
             cor != VERDE && cor != BRANCO );

   return cor; /* Retorna uma cor que consta na paleta */
}

/****
 *
 * main(): Desenha aleatoriamente na tela caracteres que
 *         representam cores
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int i, j;

      /* Alimenta o gerador de nmeros aleatrios */
   srand(time(NULL));

   putchar('\n'); /* Primeira pincelada */

      /* Pinta o quadro linha a linha */
   for (i = 0; i < VERTICAL; i++) {
      for (j = 0; j < HORIZONTAL; j++) {
         putchar(EscolheCor());
      }

      putchar('\n'); /* Passa para a prxima linha */
   }

   putchar('\n'); /* Toque final */

   return 0;
}

/******************* Exemplo de Execuo *******************

+''.+'..++. ...''
. +..' .'+. .'+. +
'...+'+.+ .'+ +'.+
 .'.'....+.+''+'.'
'..' .'.++' +.'+ +
+  '..' '+ '...  '
 .+ . ..' ++'' .++
 .'+  '' . +.' ...
  +....'+...  '.''
'+''' '+.+.. .'+'
++ +..  ++.+ +' ..
'..''++ ' '.+  ..+
.+. .' ..'.'.'' .

***********************************************************/
