/****
 *
 * Ttulo: Alinhando na Tela Valores Inteiros Lidos via Teclado
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 10/09/2012
 * ltima modificao: 10/09/2012
 *
 * Descrio: Alinha um valor inteiro lido via teclado na tela
 *
 * Entrada: v. exemplo de execuo
 *
 * Sada: v. exemplo de execuo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada          */
#include <string.h>  /* Processamento de strings */
#include "leitura.h" /* LeituraFacil             */

/***************** Constantes Simblicas ******************/

   /* Tamanho do array que armazenar os strings */
#define TAM_ARRAY  50

/****************** Definies de Tipos *******************/

   /* Tipos de alinhamento */
typedef enum {ESQUERDA, CENTRO, DIREITA} tAlinhamento;

/***************** Definies de Funes ******************/

/****
 *
 * AlinhaString(): alinha um string  esquerda,  direita
 *                 ou ao centro de um array, preenchendo
 *                 o espao restante com um caractere
 *                 recebido como parmetro
 *
 * Parmetros:
 *      ar[] (sada) - array que conter o string alinhado
 *      largura (entrada) - tamanho do array ar[]
 *      preenchimento (entrada) - caractere usado no
 *                                preenchimento do espao
 *                                restante
 *      alinhamento (entrada) - o tipo de alinhamento
 *      str (entrada) - o string que ser alinhado
 *
 * Retorno: ponteiro para o incio do array contendo o
 *          resultado da operao
 *
 * Nota: Se o comprimento do string str for maior do que o
 *       tamanho do array, ele ser truncado.
 *
 ****/
char *AlinhaString( char ar[], int largura,
                    char preenchimento,
                    tAlinhamento alinhamento,
                    const char *str )
{
   char *p;
   int   i,
         menorLargura,/* O menor valor entre o tamanho do */
                      /* array e o comprimento do string  */
         tamStr; /* Comprimento do string */

      /* Calcula o tamanho do string */
   tamStr = strlen(str);

      /* Determina a menor largura */
   menorLargura = largura < tamStr ? largura : tamStr;

      /* Preenche todo o array com o */
      /* caractere de preenchimento  */
   for (i = 0; i < largura - 1; ++i) {
      ar[i] = preenchimento;
   }

      /* Transforma o contedo do array num string */
   ar[largura - 1] = '\0';

      /* Determina a posio inicial */
      /* do string dentro do array   */
   if (alinhamento == ESQUERDA) {
      p = ar;
   } else if (alinhamento == CENTRO) {
      p = ar + (largura - menorLargura)/2;
   } else { /* alinhamento == DIREITA */
      p = ar + largura - menorLargura - 1;
   }

   /**************************************************/
   /*                                                */
   /* Neste ponto, p aponta para o elemento do array */
   /* que receber o primeiro caractere do string    */
   /*                                                */
   /**************************************************/

      /* Insere o string em sua posio */
   for (i = 0; i < menorLargura; ++i) {
      p[i] = str[i];
   }

      /* Retorna o endereo do array  */
      /* que contm o string alinhado */
   return ar;
}

/****
 *
 * main(): Alinha na tela um valor inteiro lido via teclado
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char         str[TAM_ARRAY], /* Armazenar o string a */
                                /* ser escrito na tela   */
                ar[TAM_ARRAY]; /* Armazenar o string   */
                               /* antes de ser alinhado */
   int          x, /* Inteiro que ser lido e apresentado */
                op, /* Opo escolhida pelo usurio */
                preenche; /* Caractere de preenchimento */
   tAlinhamento alinhamento; /* Opo de alinhamento */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa alinha o numero inteiro"
           "\n\t>>> de acordo com a sua preferencia.\n" );

      /* L o valor inteiro */
   printf("\n\t>>> Digite um valor inteiro: " );
   x = LeInteiro();

      /* L a opo de alinhamento do usurio */
   printf( "\n\t>>> Opcoes de alinhamento: Esquerda (E),"
           "\n\t>>> Direita (D) e Centro (C). "
           "\n\t>>> Qual e' a sua opcao? ");
   op = LeOpcao("eEdDcC");

   if (op == 'e' || op == 'E') {
      alinhamento = ESQUERDA;
   } else if (op == 'd' || op == 'D') {
      alinhamento = DIREITA;
   } else {
      alinhamento = CENTRO;
   }

      /* L o caractere de preenchimento */
      /* de espaos em branco            */
   printf("\n\t>>> Caractere de preenchimento: ");
   preenche = LeCaractere();

      /* Escreve no array o valor inteiro lido */
   sprintf(ar, "%d", x);

      /* Alinha o string antes de apresent-lo na tela */
   AlinhaString(str, TAM_ARRAY, preenche, alinhamento, ar);

      /* Apresenta o string alinhado na tela */
   printf( "\n%s\n", str );

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa alinha o numero inteiro
        >>> de acordo com a sua preferencia.

        >>> Digite um valor inteiro: 12345

        >>> Opcoes de alinhamento: Esquerda (E),
        >>> Direita (D) e Centro (C).
        >>> Qual e' a sua opcao? c

        >>> Caractere de preenchimento: +

++++++++++++++++++++++12345++++++++++++++++++++++

        >>> Obrigado por usar este programa.

**********************************************************/
