/****
 *
 * Ttulo: Abrindo Arquivos Compulsoriamente
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 30/08/2012
 * ltima modificao: 30/08/2012
 *
 * Descrio: Obriga o usurio a introduzir um nome de
 *            arquivo que possa ser aberto
 *
 * Entrada: O nome de um arquivo de texto
 *
 * Sada: Informao sobre a abertura do arquivo
 *
 ****/

#include <stdio.h>    /* Entrada e sada */
#include "leitura.h"  /* LeituraFacil    */

/****
 *
 * AbreArquivo(): L o nome de um arquivo e abre-o
 *
 * Parmetros:
 *      prompt (entrada) - prompt que ser apresentado ao usurio
 *      modo (entrada) - modo de abertura do arquivo
 *
 * Retorno: Stream associado ao arquivo aberto
 *
 ****/
FILE *AbreArquivo(const char *prompt, const char *modo)
{
      /* Array que armazenar o nome do arquivo */
   char  nomeArq[FILENAME_MAX];
   FILE *stream; /* Stream associado ao arquivo aberto */

      /* O lao s encerra quando o arquivo */
      /* indicado pelo usurio for aberto   */
   while (1) {
         /* Apresenta o prompt e l o nome do arquivo */
      printf("%s", prompt);
      LeString(nomeArq, FILENAME_MAX);

         /* Tenta abrir o arquivo */
      stream = fopen(nomeArq, modo);

         /* Se o arquivo foi aberto, o servio est completo */
      if (stream) {
         break; /* Encerra o lao */
      }

         /* Se o arquivo no foi aberto, apresenta */
         /* mensagem correspondente ao usurio     */
      printf( "\a\n\t>>> Nao foi possivel abrir o arquivo"
              " \"%s\"\n", nomeArq );
   }

   return stream;
}

/****
 *
 * main(): Obriga o usurio a introduzir um nome de arquivo
 *         que possa ser aberto
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   FILE *stream;

   printf( "\n\t>>> Este programa requer que voce introduza"
           "\n\t>>> um nome de arquivo que possa ser aberto\n" );

      /* Abre o arquivo indicado pelo usurio */
   stream = AbreArquivo( "\n\t>>> Digite o nome do"
                         " arquivo: ", "rb" );

      /* Informa o usurio que, agora, ele est liberado */
   printf( "\n\t>>> Abertura do arquivo foi bem sucedida\n" );

      /* Fecha o arquivo, j que no  */
      /* h mais nada a fazer com ele */
   fclose(stream);

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa requer que voce introduza
        >>> um nome de arquivo que possa ser aberto

        >>> Digite o nome do arquivo: Inexistente

        >>> Nao foi possivel abrir o arquivo "Inexistente"

        >>> Digite o nome do arquivo: Tudor.txt

        >>> Abertura do arquivo foi bem sucedida

**********************************************************/

